@@ -22,46 +22,16 @@
 #include "libpies.h"
 #include "grecsasrt.h"
 
-int
-assert_grecs_value_type (grecs_locus_t *locus,
-			 const grecs_value_t *value, int type)
-{
-  if (GRECS_VALUE_EMPTY_P (value))
-    {
-      grecs_error (locus, 0, _("expected %s"),
-		    grecs_data_type_string (type));
-      return 1;
-    }
-  if (value->type != type)
-    {
-      grecs_error (locus, 0, _("expected %s, but found %s"),
-		    grecs_data_type_string (type),
-		    grecs_data_type_string (value->type));
-      return 1;
-    }
-  return 0;
-}
-
-int
-assert_scalar_stmt (grecs_locus_t *locus, enum grecs_callback_command cmd)
-{
-  if (cmd != grecs_callback_set_value)
-    {
-      grecs_error (locus, 0, _("unexpected block statement"));
-      return 1;
-    }
-  return 0;
-}
-
 int
 conf_callback_url (enum grecs_callback_command cmd,
-		   grecs_locus_t *locus,
-		   void *varptr, grecs_value_t *value, void *cb_data)
+		   grecs_node_t *node,
+		   void *varptr, void *cb_data)
 {
+  grecs_locus_t *locus = &node->locus;
+  grecs_value_t *value = node->v.value;
   struct pies_url *url;
 
-  if (assert_scalar_stmt (locus, cmd)
-      || assert_grecs_value_type (locus, value, GRECS_TYPE_STRING))
+  if (grecs_assert_node_value_type (cmd, node, GRECS_TYPE_STRING))
     return 1;
   if (pies_url_create (&url, value->v.string))
     {
