@@ -674,6 +674,9 @@ component_verify (struct component *comp, grecs_locus_t *locus)
 	  return 1;
 	}
     default:
+      if (PIES_SYSVINIT_ENABLED && comp->mode >= pies_mark_sysvinit)
+	COMPERR (grecs_error,
+		 "%s", _("SystemV init support is not compiled in"));
       /* FIXME: more checks perhaps */
       break;
     }
