@@ -1,5 +1,5 @@
 /* This file is part of GNU Pies.
-   Copyright (C) 2008-2013, 2016 Sergey Poznyakoff
+   Copyright (C) 2008-2019 Sergey Poznyakoff
 
    GNU Pies is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -50,14 +50,14 @@ TC (unsigned *R, int n)
     {
       ccol = cword;
       rowj = R;
-                
+
       while (rowj < relend)
 	{
 	  if (*ccol & mask)
 	    {
 	      rp = rowi;
 	      rend = (unsigned *) ((char *) rowj + rowsize);
-                                
+
 	      while (rowj < rend)
 		*rowj++ |= *rp++;
 	    }
@@ -65,10 +65,10 @@ TC (unsigned *R, int n)
 	    {
 	      rowj = (unsigned *) ((char *) rowj + rowsize);
 	    }
-	  
+
 	  ccol = (unsigned *) ((char *) ccol + rowsize);
 	}
-                
+
       mask <<= 1;
       if (mask == 0)
 	{
@@ -81,27 +81,34 @@ TC (unsigned *R, int n)
 
 struct pies_depmap
 {
-  size_t nrows;
-  size_t rowlen;
-  unsigned r[1];
+  size_t nrows;         /* Number of rows (== number of mapped elements) */
+  size_t rowlen;        /* Length of a row in words */
+  unsigned r[1];        /* Data rows */
 };
-  
+
 pies_depmap_t
 depmap_alloc (size_t count)
 {
   size_t size = (count + BITS_PER_WORD - 1) / BITS_PER_WORD;
-  pies_depmap_t dmap = grecs_zalloc (sizeof (*dmap) - 1 
+  pies_depmap_t dmap = grecs_zalloc (sizeof (*dmap) - 1
 				     + count * size * sizeof (unsigned));
   dmap->nrows  = count;
   dmap->rowlen = size;
   return dmap;
 }
 
+/* Return size of a depmap row in bytes */
+static inline size_t
+depmap_row_size (pies_depmap_t dpm)
+{
+  return dpm->rowlen * sizeof (unsigned);
+}
+
 pies_depmap_t
 depmap_copy (pies_depmap_t dpm)
 {
   pies_depmap_t copy = depmap_alloc (dpm->nrows);
-  memcpy (copy->r, dpm->r, dpm->nrows * dpm->rowlen * sizeof (unsigned));
+  memcpy (copy->r, dpm->r, dpm->nrows * depmap_row_size (dpm));
   return copy;
 }
 
@@ -151,9 +158,9 @@ depmap_next (pies_depmap_t dmap, pies_depmap_pos_t pos)
        pos->coord[pos->dir]++)
     if (depmap_isset (dmap, pos->coord[0], pos->coord[1]))
       break;
-  
+
   return pos->coord[pos->dir] == dmap->nrows ?
-             (size_t) -1 : pos->coord[pos->dir];
+	     (size_t) -1 : pos->coord[pos->dir];
 }
 
 size_t
@@ -174,21 +181,22 @@ depmap_end (pies_depmap_pos_t pos)
   grecs_free (pos);
 }
 
+/* Remove nth row and column from the map */
 void
-depmap_clear_all (pies_depmap_t dmap, enum pies_depmap_direction dir,
-		  size_t coord)
+depmap_remove (pies_depmap_t dmap, size_t n)
 {
-  size_t i;
-  
-  switch (dir)
+  if (n < dmap->nrows - 1)
     {
-    case depmap_row:
-      for (i = 0; i < dmap->nrows; i++)
-	depmap_clear (dmap, coord, i);
-      break;
+      size_t i, j;
 
-    case depmap_col:
+      /* Remove nth row */
+      memmove (depmap_rowptr (dmap, n), depmap_rowptr (dmap, n + 1),
+	       (dmap->nrows - n - 1) * depmap_row_size (dmap));
+      /* Renumber all columns j >= n: j = j - 1 */
       for (i = 0; i < dmap->nrows; i++)
-	depmap_clear (dmap, i, coord);
+	for (j = n; j < dmap->nrows - 1; j++)
+	  (depmap_isset (dmap, i, j + 1) ? depmap_set : depmap_clear)
+	    (dmap, i, j);
     }
+  dmap->nrows--;
 }
