@@ -80,10 +80,12 @@ static struct grecs_keyword instance_keywords[] = {
 
 static int
 callback_instance (enum grecs_callback_command cmd,
-		   grecs_locus_t *locus,
-		   void *varptr,
-		   grecs_value_t *value, void *cb_data)
+		   grecs_node_t *node,
+		   void *varptr, void *cb_data)
 {
+  grecs_locus_t *locus = &node->locus;
+  grecs_value_t *value = node->v.value;
+
   switch (cmd)
     {
     case grecs_callback_section_begin:
@@ -92,7 +94,7 @@ callback_instance (enum grecs_callback_command cmd,
 	  grecs_error (locus, 0, _("missing tag"));
 	  return 1;
 	}
-      if (assert_grecs_value_type (&value->locus, value, GRECS_TYPE_STRING))
+      if (grecs_assert_value_type (value, GRECS_TYPE_STRING, locus))
 	return 1;
       if (strcmp (value->v.string, instance) == 0)
 	*(struct client_conn **) cb_data = &client;
@@ -184,8 +186,10 @@ parse_config (void)
 	  node = grecs_find_node (tree, "/control/socket");
 	  if (node)
 	    {
-	      assert_grecs_value_type (&node->locus,
-				       node->v.value, GRECS_TYPE_STRING);
+	      if (grecs_assert_value_type (node->v.value, GRECS_TYPE_STRING,
+					   &node->locus))
+		  exit (EX_CONFIG);
+
 	      if (pies_url_create (&client.url, node->v.value->v.string))
 		{
 		  grecs_error (&node->locus, 0, _("%s: cannot create URL: %s"),
@@ -411,123 +415,22 @@ static struct shttp_connection *
 shttp_connect (struct pies_url *url, struct grecs_sockaddr *source_addr)
 {
   int fd;
-  union pies_sockaddr_storage addr;
-  socklen_t socklen;
-  int flags;
+  struct pies_url *conn_url;
   struct shttp_connection *conn;
   FILE *fp;
+
+  if (pies_url_copy (&conn_url, url))
+    grecs_alloc_die ();
   
-  if (strcmp (url->scheme, "unix") == 0
-      || strcmp (url->scheme, "file") == 0
-      || strcmp (url->scheme, "socket") == 0)
+  if (conn_url->port == 0 && strcmp (conn_url->scheme, "inet") == 0)
     {
-      struct stat st;
-		
-      if (url->port)
-	{
-	  grecs_error (NULL, 0, _("%s: invalid connection type: "
-				  "port is meaningless for UNIX sockets"),
-		       url->string);
-	  return NULL;
-	}
-		
-      if (strlen (url->path) > sizeof addr.s_un.sun_path)
-	{
-	  errno = EINVAL;
-	  grecs_error (NULL, 0,
-		       _("%s: UNIX socket name too long"),
-		       url->path);
-	  return NULL;
-	}
-		
-      addr.s.sa_family = PF_UNIX;
-      socklen = sizeof (addr.s_un);
-      strcpy (addr.s_un.sun_path, url->path);
-		
-      if (stat (url->path, &st))
-	{
-	  if (errno != ENOENT)
-	    {
-	      grecs_error (NULL, errno, _("%s: %s failed"), url->path, "stat");
-	      return NULL;
-	    }
-	}
-      else
-	{
-	  if (!S_ISSOCK (st.st_mode))
-	    {
-	      grecs_error (NULL, 0, _("%s: not a socket"), url->path);
-	      return NULL;
-	    }
-	}
+      conn_url->port_s = grecs_strdup ("8080");
+      conn_url->port = 8080;
     }
-  else if (strcmp (url->scheme, "inet") == 0)
-    {
-      short pnum;
-      struct hostent *hp;
-      
-      addr.s_in.sin_family = PF_INET;
-      socklen = sizeof (addr.s_in);
-
-      pnum = url->port ? url->port : 8080;
-      
-      hp = gethostbyname (url->host);
-      if (!hp)
-	{
-	  grecs_error (NULL, 0, _("%s: unknown host name"), url->string);
-	  return NULL;
-	}
-
-      addr.s_in.sin_family = hp->h_addrtype;
-      switch (hp->h_addrtype)
-	{
-	case AF_INET:
-	  memmove (&addr.s_in.sin_addr, hp->h_addr, 4);
-	  addr.s_in.sin_port = htons (pnum);
-	  break;
-	  
-	default:
-	  grecs_error (NULL, 0,
-		       _("%s: invalid connection type: "
-			 "unsupported address family"),
-		       url->string);
-	  return NULL;
-	}
-    }
-  else
-    {
-      grecs_error (NULL, 0, _("%s: unsupported protocol"), url->string);
-      return NULL;
-    }
-
-  fd = socket (addr.s.sa_family, SOCK_STREAM, 0);
+  fd = url_connect (conn_url, source_addr);
   if (fd == -1)
     {
-      grecs_error (NULL, errno, _("%s: %s failed"), url->string, "socket");
-      return NULL;
-    }
-  
-  if ((flags = fcntl (fd, F_GETFD, 0)) == -1
-      || fcntl (fd, F_SETFD, flags | FD_CLOEXEC) == -1)
-    grecs_error (NULL, 0,  _("%s: cannot set close-on-exec: %s"),
-		 url->string, strerror (errno));
-
-  if (source_addr)
-    {
-      if (source_addr->sa->sa_family != addr.s.sa_family)
-	grecs_error (NULL, 0,
-		     _("source and destination address family differ"));
-      else if (bind (fd, source_addr->sa, source_addr->len) < 0)
-	{
-	  grecs_error (NULL, errno, _("%s: %s failed"), url->string, "bind");
-	  exit (EX_UNAVAILABLE);
-	}
-    }
-  
-  if (connect (fd, &addr.s, socklen))
-    {
-      grecs_error (NULL, errno, _("%s: %s failed"), url->string, "connect");
-      close (fd);
+      pies_url_destroy (&conn_url);
       return NULL;
     }
 
@@ -541,8 +444,7 @@ shttp_connect (struct pies_url *url, struct grecs_sockaddr *source_addr)
 
   conn = grecs_zalloc (sizeof (*conn));
   conn->fp = fp;
-  if (pies_url_copy (&conn->url, url))
-    grecs_alloc_die ();
+  conn->url = conn_url;
 
   if (!no_netrc_option)
     netrc_scan (conn->url);
@@ -1372,7 +1274,7 @@ pcond_parse_unary (struct cmdline_parser_state *state, struct json_value **ret)
       pcond_parse_or (state, ret);
       term = next_token (state);
       if (!term || term[0] != ')')
-	parse_error ("%s", _("unbalanced parentesis"));
+	parse_error ("%s", _("unbalanced parenthesis"));
     }
   else
     parse_error (_("parse error at %s"), term);
