@@ -824,7 +824,7 @@ prog_start_prologue (struct prog *prog)
 	environ_unset (prog->v.p.env, sockenv_var[i], NULL);
     }
   envop_exec (prog->v.p.comp->envop, prog->v.p.env);
-  if (init_process)
+  if (SYSVINIT_ACTIVE)
     {
       size_t i;
       for (i = 0; sysvinit_environ_hint[i]; i++)
@@ -899,7 +899,7 @@ prog_start (struct prog *prog)
 
   if (is_sysvinit (prog->v.p.comp))
     {
-      if (!init_process)
+      if (!SYSVINIT_ACTIVE)
 	{
 	  if (prog->active)
 	    {
@@ -995,7 +995,7 @@ prog_start (struct prog *prog)
 	  break;
 
 	default:
-	  if (init_process)
+	  if (SYSVINIT_ACTIVE)
 	    {
 	      int fd = console_open (O_RDWR|O_NOCTTY);
 	      if (fd < 0)
@@ -1037,7 +1037,7 @@ prog_start (struct prog *prog)
 	  break;
 	}
 
-      if (!init_process)
+      if (!SYSVINIT_ACTIVE)
 	{
 	  if (redir[RETR_ERR] == -1)
 	    {
@@ -1501,10 +1501,7 @@ prog_start_prerequisites (struct prog *prog)
 	}
 
       if (!prog->active)
-	{
-	  prog->active = 0;
-	  return 1;
-	}
+	return 1;
 
       p = comp->prog;
       switch (p->v.p.status)
@@ -1600,7 +1597,7 @@ print_status (const char *tag, pid_t pid, int status, int expect_term)
 {
   int prio;
 
-  if (init_process)
+  if (SYSVINIT_ACTIVE)
     {
       if (debug_level <= 1)
 	return;
@@ -2147,7 +2144,8 @@ progman_cleanup (int expect_term)
 	    }
 	  else
 	    {
-	      if (prog->v.p.comp->mode >= pies_mark_sysvinit
+	      if (PIES_SYSVINIT_ENABLED
+		  && prog->v.p.comp->mode >= pies_mark_sysvinit
 		  && prog->v.p.comp->mode != pies_comp_ondemand)
 		{
 		  sysvinit_acct (SYSV_ACCT_PROC_STOP, "", prog_tag (prog),
@@ -2303,7 +2301,7 @@ prog_to_stop (struct prog *prog)
 }
 
 #define DIAG_CON \
-  (init_process ? (DIAG_TO_STDERR|DIAG_REOPEN_LOG) : diag_output)
+  (SYSVINIT_ACTIVE ? (DIAG_TO_STDERR|DIAG_REOPEN_LOG) : diag_output)
 
 /* Stop all program components marked for termination.  Wait at most
    2*shutdown_timeout seconds. */
